
class NotAllowedToSpeakError(Exception):
    def __init__(self, config_path, message=None):
        self.config_path = config_path
        self.message = message or f"System is not allowed to speak. Open `{self.config_path}` and set `tts.is_allowed` to `True`."
        super().__init__(self.message)


import os, sys
import asyncio
from say.main import main


USERNAME = os.environ.get('USERNAME', 'Unknown')
HOME = "/home/%s" % USERNAME
I18N, L10N = (x for x in os.environ.get('LANG', "en_EN.UTF-8").split(".")[0].split("_"))

def run():
    import argparse

    def str2bool(v):
        if isinstance(v, bool):
            return v
        if v.lower() in ("oui", "yes", "true", "t", "o", "y", "1"):
            return True
        if v.lower() in ("non", "no", "false", "f", "n", "0"):
            return False
        raise argparse.ArgumentTypeError("Boolean value expected.")

    parser = argparse.ArgumentParser(description="Same as echo but with Text-To-Speech.")

    # This is here so that you can alias echo to say in your favorite shell.
    # $ alias echo='say'
    parser.add_argument('-n', '--n', action="store_true", help="do not output the trailing newline")
    parser.add_argument('-e', '--e', action="store_true", help="enable interpretation of backslash escapes")
    parser.add_argument('-E', '--E', action="store_true", help="disable interpretation of backslash escapes (default)")
    parser.add_argument('-v', '--version', action='store_true', help="output version information and exit")
    # This is here for the TTS client
    parser.add_argument('-L', '--lang', type=str, help=("Language to be spoken (default: $LANG)"), default=I18N)
    parser.add_argument(
        "--out_path",
        type=str,
        required=False,
        help="Output wav file path.",
    )
    # This is here so you can choose the TTS and vocoder models for the server.
    parser.add_argument(
        "--list_models",
        type=str2bool,
        nargs="?",
        const=True,
        default=False,
        help="list available pre-trained tts and vocoder models.",
    )
    parser.add_argument(
        "--model_name",
        type=str,
        default="tts_models/multilingual/multi-dataset/your_tts",
        help="Name of one of the pre-trained tts models in format <language>/<dataset>/<model_name>",
    )
    parser.add_argument("--vocoder_name", type=str, default=None, help="name of one of the released vocoder models.")

    # Args for running custom models
    parser.add_argument("--config_path", default=None, type=str, help="Path to model config file.")
    parser.add_argument(
        "--model_path",
        type=str,
        default=None,
        help="Path to model file.",
    )
    parser.add_argument(
        "--vocoder_path",
        type=str,
        help="Path to vocoder model file. If it is not defined, model uses GL as vocoder. Please make sure that you installed vocoder library before (WaveRNN).",
        default=None,
    )
    parser.add_argument("--vocoder_config_path", type=str, help="Path to vocoder model config file.", default=None)
    # parser.add_argument(
    #     "--language_idx",
    #     type=str,
    #     help="Target language ID for a multi-lingual TTS model.",
    #     default=None,
    # )
    # parser.add_argument(
    #     "--list_language_idxs",
    #     help="List available language ids for the defined multi-lingual model.",
    #     type=str2bool,
    #     nargs="?",
    #     const=True,
    #     default=False,
    # )
    parser.add_argument(
        "--speaker_idx",
        type=str,
        help="Target speaker ID for a multi-speaker TTS model.",
        default=None,
    )
    # parser.add_argument(
    #     "--list_speaker_idxs",
    #     help="List available speaker ids for the defined multi-speaker model.",
    #     type=str2bool,
    #     nargs="?",
    #     const=True,
    #     default=False,
    # )
    parser.add_argument(
        "--speaker_wav",
        nargs="+",
        help="wav file(s) to condition a multi-speaker TTS model with a Speaker Encoder. You can give multiple file paths. The d_vectors is computed as their average.",
        default=None,
    )
    parser.add_argument("--speakers_file_path", type=str, help="JSON file for multi-speaker model.", default=None)
    parser.add_argument("--use_cuda", type=str2bool, default=False, help="true to use CUDA.")
    parser.add_argument("--debug", type=str2bool, default=False, help="true to enable debug mode.")
    
    # This is really what we need.
    parser.add_argument('text', type=str, help="Text to be said.", nargs="*")
    ARGS = parser.parse_args()
    try:
        main(ARGS)
        sys.exit(0)
    except KeyboardInterrupt:
        sys.exit(1)

if __name__ == '__main__':
   run()


from say.TTS import as_client, utils, exception

import sys, os
import asyncio

def main(ARGS):
    exit_code = 0
    try:

        si, sw = utils.get_speaker(ARGS.speaker_idx, ARGS.speaker_wav)

        if ARGS.version:
            as_client.say_version(ARGS.lang, speaker_idx=si, style_wav=sw)
            sys.exit(exit_code)

        elif ARGS.text:
            _text = [" ".join(ARGS.text),]
            _show_version = False
            _no_newline = False
            if ARGS.e:
                _enable_interpretation = True
                _disable_interpretation = False
            else:
                _enable_interpretation = False
                _disable_interpretation = True

            if ARGS.lang == "fr":
                lang = "fr-fr"
            elif ARGS.lang == "en":
                lang = ARGS.lang
            else:
                lang = ARGS.lang
            
            asyncio.run(as_client._say(_text, lang, speaker_idx=si, style_wav=sw, save_output=ARGS.out_path, show_version=_show_version, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline))
            sys.exit(exit_code)
        else:
            _text = [
                "No attribute `text`.",
                "say --help",
                "For more information.",
                "Starting server now.",
                "Please wait."
            ]
            _show_version = False
            _enable_interpretation = True
            _disable_interpretation = False
            _no_newline = False
            for t in _text:
                utils.echo(t, show_version=_show_version, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline)
            
            from say.TTS.as_service import as_service

            if ARGS.list_models:
                utils.manager.list_models()
                sys.exit(exit_code)
            
            # Check if conf exist
            CONFIG = utils.get_config_or_default()

            is_allowed_to_speak = utils.is_allowed_to_speak(CONFIG)
            
            if not is_allowed_to_speak:
                raise exception.NotAllowedToSpeakError(utils.CONFIG_PATH)

            model_name = utils.get_models_name(ARGS.model_name, CONFIG)
            

            # update in-use models to the specified released models.
            model_path = None
            config_path = None
            speakers_file_path = None
            vocoder_path = None
            vocoder_config_path = None

            if model_name is not None and not ARGS.model_path:
                model_path, config_path, model_item = utils.manager.download_model(model_name)
                ARGS.vocoder_name = model_item["default_vocoder"] if ARGS.vocoder_name is None else ARGS.vocoder_name
            
            if ARGS.vocoder_name is not None and not ARGS.vocoder_path:
                vocoder_path, vocoder_config_path, _ = utils.manager.download_model(ARGS.vocoder_name)
            
            if ARGS.model_path is not None:
                model_path = ARGS.model_path
                config_path = ARGS.config_path
                speakers_file_path = ARGS.speakers_file_path

            if ARGS.vocoder_path is not None:
                vocoder_path = ARGS.vocoder_path
                vocoder_config_path = ARGS.vocoder_config_path
            
            app = as_service(model_path, config_path, speakers_file_path, vocoder_path, vocoder_config_path, ARGS.use_cuda)

            app.run(
                host=CONFIG["service"]["host"],
                port=CONFIG["service"]["port"],
                debug=ARGS.debug,
            )
    except Exception as err:
        exit_code = 1
        raise err
    
    sys.exit(exit_code)


from say.TTS.engine import TTS, Response, Error, load_models
from say.TTS import utils

import os
import json
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path
from time import perf_counter
import toml
from sanic import Sanic, response
from sanic.log import logger

def as_service(
    models_path,
    config_path,
    speakers_file_path,
    vocoder_path,
    vocoder_config_path,
    use_cuda,
    tts_languages_file=None,
    encoder_checkpoint="",
    encoder_config="",
    debug=False,
):
    
    CONFIG = utils.get_config_or_default()

    # Load app configs and initialize STT model
    try:
        n_proc_available = CONFIG['service']['n_proc'] or utils.get_available_cpu_count()
    except Exception as e:
        print(e)
        print("Using 2 instead.")
        n_proc_available = 2

    synthesizer, use_multi_speaker, speaker_manager, use_gst = load_models(
            models_path,
            config_path,
            speakers_file_path,
            vocoder_path,
            vocoder_config_path,
            use_cuda,
            tts_languages_file=None,
            encoder_checkpoint="",
            encoder_config="",
        )

    engine = TTS(
        synthesizer, use_multi_speaker, speaker_manager, use_gst
    )

    # Initialze Sanic and ThreadPoolExecutor
    executor = ThreadPoolExecutor(max_workers=n_proc_available)
    app = Sanic("tts_service")


    @app.route("/", methods=["GET"])
    async def healthcheck(_):
        return response.text("Welcome to say.sock: TTS as a Service!")


    @app.websocket("/api/v1/tts")
    async def tts(request, ws):
        logger.debug(f"Received {request.method} request at {request.path}")
        try:
            _text = await ws.recv()
            json_data = json.loads(_text.decode('utf-8', 'ignore'))
            logger.debug("With parameters:")
            logger.debug(json_data)
            inference_start = perf_counter()
            wav = await app.loop.run_in_executor(executor, lambda: engine.run(json_data.get('text'), json_data.get('language'), json_data.get('speaker_idx'), json_data.get('style_wav')))
            inference_end = perf_counter() - inference_start
            logger.debug(f"Completed {request.method} request at {request.path} in {inference_end} seconds")
            await ws.send(wav.getvalue())
            logger.debug("Sent audio to client")
            
        except Exception as e:  # pylint: disable=broad-except
            logger.debug(f"Failed to process {request.method} request at {request.path}. The exception is: {str(e)}.")
            await ws.send(b"Something went wrong")
            raise e
        finally:
            await ws.close()
    
    return app

if __name__ == '__main__':
    from say.entry_points import run_say
    run_say.run()


from say import __version__

from TTS.utils.manage import ModelManager

import os
import asyncio
import urllib.request
import re
import json
import toml
import logging
import threading


from pathlib import Path
from python_shell import Shell
from python_shell.util.streaming import decode_stream

from say import *

def get_config_or_default():
    # Check if conf exist

    if os.path.exists(CONFIG_PATH):
        with open(CONFIG_PATH, 'r') as cfg:
            CONFIG = toml.loads(cfg.read())
    else:
        CONFIG = {
            'service': {
                'host': '0.0.0.0',
                'port': '5067'
            },
            'tts': {
                'models': "tts_models/multilingual/multi-dataset/your_tts",
                'language': "fr-fr" if I18N == "fr" else I18N,
                'speaker_wav': f"{ASSISTANT_PATH}/data/{I18N}/TTS/styles/default.wav",
                'is_allowed': False
            }
        }
        Path.mkdir(Path(CONFIG_PATH).parent, exist_ok=True)
        with open(CONFIG_PATH, 'w') as f:
            f.write(toml.dumps(CONFIG))
    
    return CONFIG

def download_speaker(output_file, lang="en", gender="male", name="default"):
    default_speaker = f"https://gitlab.com/waser-technologies/data/tts/{lang}/voices/-/raw/master/{gender}/{name}.wav"
    urllib.request.urlretrieve(default_speaker, output_file)
    return output_file

def get_speaker(idx=None, wav=None, conf=get_config_or_default()):
    # Get a speaker id or speaker wav
    speaker_id = None
    speaker_wav = None

    if idx:
        speaker_id = idx[0]
    elif wav:
        speaker_wav = wav[0]
    elif conf.get('tts'):
        speaker_id = conf['tts'].get('speaker_id', None)
        speaker_wav = conf['tts'].get('speaker_wav', None)
    
    return speaker_id, speaker_wav

def get_models_name(model_name=None, conf=get_config_or_default()):
    """
    Makes sure Config represent loaded models name.
    """
    _tts_conf = conf.get('tts', None)
    if _tts_conf:
        _tts_models_name = _tts_conf.get('models', None)
        if _tts_models_name:
            if model_name != _tts_models_name:
                conf['tts']['models'] = model_name
                Path.mkdir(Path(CONFIG_PATH).parent, exist_ok=True)
                with open(CONFIG_PATH, 'w') as f:
                    f.write(toml.dumps(conf))
    return model_name

def is_allowed_to_speak(conf=get_config_or_default()):
    _tts_conf = conf.get('tts', False)
    if _tts_conf:
        return _tts_conf.get('is_allowed', False)
    return False

def get_loc_model_path():
	"""
    Get localised models path.
    """
	return Path(__TTS_file__).parent / ".models.json"

manager = ModelManager()

def echo(text="", show_version=False, enable_interpretation=False, disable_interpretation=True, no_newline=False, end="\n"):
    if text:
        if enable_interpretation:
            e = Shell.echo('-e', text)
        else:
            e = Shell.echo(text)
    
        p = decode_stream(e.output)
        print(str(p), end="")
        return p

def get_available_cpu_count():
    """Number of available virtual or physical CPUs on this system, i.e.
    user/real as output by time(1) when called with an optimally scaling
    userspace-only program
    See this https://stackoverflow.com/a/1006301/13561390"""

    # cpuset
    # cpuset may restrict the number of *available* processors
    try:
        m = re.search(r"(?m)^Cpus_allowed:\s*(.*)$", open("/proc/self/status").read())
        if m:
            res = bin(int(m.group(1).replace(",", ""), 16)).count("1")
            if res > 0:
                return res
    except IOError:
        pass

    # Python 2.6+
    try:
        import multiprocessing

        return multiprocessing.cpu_count()
    except (ImportError, NotImplementedError):
        pass

    # https://github.com/giampaolo/psutil
    try:
        import psutil

        return psutil.cpu_count()  # psutil.NUM_CPUS on old versions
    except (ImportError, AttributeError):
        pass

    # POSIX
    try:
        res = int(os.sysconf("SC_NPROCESSORS_ONLN"))

        if res > 0:
            return res
    except (AttributeError, ValueError):
        pass

    # Windows
    try:
        res = int(os.environ["NUMBER_OF_PROCESSORS"])

        if res > 0:
            return res
    except (KeyError, ValueError):
        pass

    # jython
    try:
        from java.lang import Runtime

        runtime = Runtime.getRuntime()
        res = runtime.availableProcessors()
        if res > 0:
            return res
    except ImportError:
        pass

    # BSD
    try:
        sysctl = subprocess.Popen(["sysctl", "-n", "hw.ncpu"], stdout=subprocess.PIPE)
        scStdout = sysctl.communicate()[0]
        res = int(scStdout)

        if res > 0:
            return res
    except (OSError, ValueError):
        pass

    # Linux
    try:
        res = open("/proc/cpuinfo").read().count("processor\t:")

        if res > 0:
            return res
    except IOError:
        pass

    # Solaris
    try:
        pseudoDevices = os.listdir("/devices/pseudo/")
        res = 0
        for pd in pseudoDevices:
            if re.match(r"^cpuid@[0-9]+$", pd):
                res += 1

        if res > 0:
            return res
    except OSError:
        pass

    # Other UNIXes (heuristic)
    try:
        try:
            dmesg = open("/var/run/dmesg.boot").read()
        except IOError:
            dmesgProcess = subprocess.Popen(["dmesg"], stdout=subprocess.PIPE)
            dmesg = dmesgProcess.communicate()[0]

        res = 0
        while "\ncpu" + str(res) + ":" in dmesg:
            res += 1

        if res > 0:
            return res
    except OSError:
        pass

    raise Exception("Can not determine number of CPUs on this system")




import os

__version__ = "2.2.6"

I18N, L10N = (x for x in os.environ.get('LANG', "en_EN.UTF-8").split(".")[0].split("_"))

USERNAME = os.environ.get("USERNAME", 'root')
HOME = f"/home/{USERNAME}" if USERNAME != "root" else "/root"
ASSISTANT_PATH = f"{HOME}/.assistant" if USERNAME != "root" else "/usr/share/assistant"
CONFIG_PATH = f"{ASSISTANT_PATH}/tts.toml"


import os

from say import HOME

def enable_service_now():
    if not os.path.isfile("/usr/lib/systemd/user/speak.service"):
        print("No listen service present.")
        print("To download it type:")
        print("    wget https://gitlab.com/waser-technologies/technologies/say/-/raw/main/speak.service.example")
        print("    mv speak.service.example /usr/lib/systemd/user/speak.service")
        sys.exit(1)
    if not os.path.exists(f"{HOME}/.config/systemd/user/default.target.wants/assistant.listen.service"):
        print("Say TTS service is disabled.")
        print("To enable it type:")
        print(f"systemctl --user enable --now speak")
        sys.exit(1)



import setuptools

from say import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tts-say",
    version=__version__,
    author="Danny Waser",
    author_email="danny@waser.tech",
    description="echo but with TTS.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/waser-technologies/technologies/say",
    project_urls={
        "Documentation": "https://gitlab.com/waser-technologies/technologies/say/blob/main/README.md",
        "Code": "https://gitlab.com/waser-technologies/technologies/say",
        "Issue tracker": "https://gitlab.com/waser-technologies/technologies/say/issues",
    },
    packages=setuptools.find_packages('.'),
    python_requires=">=3.7,<=3.10.9",
    install_requires = [
        'TTS',
        'python_shell',
        'simpleaudio',
        'pydub',
        'num2words',
    ],
    entry_points={
        'console_scripts': [
            'say = say.entry_points.run_say:run',
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import os
import socket
import json
import websockets # not websocket!!!
import asyncio
from datetime import date
from pydub import AudioSegment
from pydub.playback import play
from num2words import num2words

from say import __version__
from say.TTS import utils

from TTS import __version__ as __TTS_version__

HOST, PORT = "localhost", "5067"
CONFIG = utils.get_config_or_default()

if CONFIG.get('service'):
    HOST = CONFIG['service'].get('host', HOST)
    PORT = CONFIG['service'].get('port', PORT)

def pronounce(wav):
	play(wav)

async def tts(text: str, language, speaker_idx="", style_wav="", host=HOST, port=PORT, save_output=None):
    async with websockets.connect(f"ws://{host}:{port}/api/v1/tts") as ws:
        try:
            j = {
                'text': text,
                'speaker_idx': speaker_idx,
                'style_wav': style_wav,
                'language': language
                }
            await ws.send(json.dumps(j).encode('utf-8', 'ignore'))
            wav = await ws.recv()
            try:
                _wav = AudioSegment(data=wav, sample_width=2, frame_rate=16000, channels=1)
                pronounce(_wav)
                if save_output:
                    _wav.export(save_output, format="wav")
            except Exception as e:
                raise Exception(e)
        except ConnectionRefusedError as e:
            pass
        except Exception as e:
            raise e
        finally:
            await ws.close()

async def _say(text: list[str], language: str, speaker_idx: str = "", style_wav: str = "", save_output: str = "", show_version: bool = False, enable_interpretation: bool = True, disable_interpretation: bool = False, no_newline: bool = False) -> list[str]:
    if style_wav and not os.path.exists(style_wav):
        utils.download_speaker(style_wav)
    
    for _text in text:
        utils.echo(text=_text, show_version=show_version, enable_interpretation=enable_interpretation, disable_interpretation=disable_interpretation, no_newline=no_newline)
        try:
            await tts([_text,], language, speaker_idx=speaker_idx, style_wav=style_wav, save_output=save_output)
        except (ConnectionRefusedError, OSError) as e:
            pass # Server is not active or something
        except Exception as e:
            raise e

    return text

def inflect_version(version, lang, dot="dot"):
    """
    Uses `num2words` to inflect version numbers to say.
    """
    Major, Minor, Bugs = version.split(".")
    _maj = num2words(int(Major), lang=lang)
    _min = num2words(int(Minor), lang=lang)
    _b = num2words(int(Bugs), lang=lang)

    return f"{_maj} {dot}, {_min} {dot}, {_b}"

def say_version(lang, speaker_idx="", style_wav=""):
    if lang == "en":
        _text = [
            f"Say, version {inflect_version(__version__, 'en', dot='dot')}.",
        ]
        language = "en"
    elif lang == "fr":
        _text = [
            f"Dit, version {inflect_version(__version__, 'fr', dot='point')}.",
        ]
        language = "fr-fr"
    else:
        raise NotImplementedError(f"Language {lang} not implemented.")
    
    _show_version = True
    _enable_interpretation = True
    _disable_interpretation = False
    _no_newline = False

    asyncio.run(_say(_text, language, speaker_idx=speaker_idx, style_wav=style_wav, show_version=False, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline))
    utils.echo(f"Say: version {str(__version__)}", show_version=False, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline)
    utils.echo(f"Copyright © {str(date.today().year)}, Danny Waser", show_version=False, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline)
    utils.echo(f"TTS version {__TTS_version__}", show_version=False, enable_interpretation=_enable_interpretation, disable_interpretation=_disable_interpretation, no_newline=_no_newline)
    os.system("/usr/bin/echo --version")


import os
import io
import glob
import logging
import json
import re

import numpy as np

#import TTS Model

from timeit import default_timer as timer
from num2words import num2words

from say.TTS import audio, utils
from TTS.utils.synthesizer import Synthesizer

def load_models(
        model_path,
        config_path,
        speakers_file_path,
        vocoder_path,
        vocoder_config_path,
        use_cuda,
        tts_languages_file=None,
        encoder_checkpoint="",
        encoder_config="",
    ):
    """
    Loads TTS models in memory.
    Returns: List[synthesizer, use_multi_speaker, speaker_manager, use_gst]
    """
    model_load_start = timer()
    synthesizer = Synthesizer(
        tts_checkpoint=model_path,
        tts_config_path=config_path,
        tts_speakers_file=speakers_file_path,
        tts_languages_file=None,
        vocoder_checkpoint=vocoder_path,
        vocoder_config=vocoder_config_path,
        encoder_checkpoint="",
        encoder_config="",
        use_cuda=use_cuda,
    )
    model_load_end = timer() - model_load_start
    logging.debug("Loaded synthesizer in %0.3fs." % (model_load_end))

    speaker_manager_load_start = timer()
    use_multi_speaker = hasattr(synthesizer.tts_model, "num_speakers") and (
        synthesizer.tts_model.num_speakers > 1 or synthesizer.tts_speakers_file is not None
    )

    speaker_manager = getattr(synthesizer.tts_model, "speaker_manager", None)
    # TODO: set this from SpeakerManager
    use_gst = synthesizer.tts_config.get("use_gst", False)
    speaker_manager_load_end = timer() - speaker_manager_load_start
    logging.debug('Loaded speaker manager in %0.3fs.' % (speaker_manager_load_end))

    return [synthesizer, use_multi_speaker, speaker_manager, use_gst]


class TTS:

    def __init__(self, synthesizer, use_multi_speaker, speaker_manager, use_gst):
        self.synthesizer = synthesizer
        self.use_multi_speaker = use_multi_speaker
        self.speaker_manager = speaker_manager
        self.use_gst = use_gst
    
    def style_wav_uri_to_dict(self, style_wav):
        """Transform an uri style_wav, in either a string (path to wav file to be use for style transfer)
        or a dict (gst tokens/values to be use for styling)
        Args:
            style_wav (str): uri
        Returns:
            Union[str, dict]: path to file (str) or gst style (dict)
        """
        if style_wav:
            if os.path.isfile(style_wav) and style_wav.endswith(".wav"):
                return style_wav  # style_wav is a .wav file located on the server

            style_wav = json.loads(style_wav)
            return style_wav  # style_wav is a gst dictionary with {token1_id : token1_weigth, ...}
        return None

    def fetch_floats_from_str(self, text):
        l = []
        for t in text.split():
            try:
                l.append(float(t))
            except ValueError:
                pass
        return l

    def convert_num2words(self, sentences: str, language):
        _sentences = sentences
        if re.search(r"[0-9]", _sentences) is not None:
            nums = self.fetch_floats_from_str(_sentences)
            for num in nums:
                n = str(num)
                w = num2words(num, lang=language)
                print(f"Found number: {n}")
                print(f"Replacing it with: {w}")
                _sentences = _sentences.replace(n, w)
                _sentences = _sentences.replace(str(int(num)), w)
        print(f"Converted {_sentences=}")
        return _sentences

    '''
    Run Inference on input audio
    '''
    def tts(self, text: str, language, speaker_idx, style_wav):
        # Run TTS
        style_wav = self.style_wav_uri_to_dict(style_wav)
        wavs = self.synthesizer.tts(text, language_name=language, speaker_wav=style_wav)
        out = io.BytesIO()
        self.synthesizer.save_wav(wavs, out)
        
        return out

    def run(self, text: list[str], language, speaker_idx, style_wav):
        _t = self.convert_num2words(" ".join(text), language.split("-")[0])
        if language.lower() == "fr":
            language = "fr-fr"
        
        return self.tts(_t, language, speaker_idx, style_wav)

class Response:
    def __init__(self, audio_bin):
        self.wav_bytes = audio_bin
    
    def to_bytes(self):
        return self.wav_bytes


class Error:
    def __init__(self, message):
        self.message = message
    
    def to_bytes(self):
        return self.message.encode('utf-8')

